@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -1206,4 +1206,141 @@ unsigned int net_field_length_size(const unsigned char *pos);
 #define NULL_LENGTH ((unsigned long)~0) /**< For ::net_store_length() */
 #define MYSQL_STMT_HEADER 4
 #define MYSQL_LONG_DATA_HEADER 6
+
+/* clang-format off */
+/**
+  Describes the current state of Asynchronous connection phase state machine
+
+  @startuml
+  [*] --> CONNECT_STAGE_INVALID
+  [*] --> CONNECT_STAGE_NOT_STARTED
+
+  CONNECT_STAGE_NOT_STARTED --> CONNECT_STAGE_NET_BEGIN_CONNECT
+  CONNECT_STAGE_NOT_STARTED --> CONNECT_STAGE_COMPLETE
+
+  CONNECT_STAGE_NET_BEGIN_CONNECT --> CONNECT_STAGE_NET_WAIT_CONNECT
+  CONNECT_STAGE_NET_BEGIN_CONNECT --> CONNECT_STAGE_NET_COMPLETE_CONNECT
+  CONNECT_STAGE_NET_BEGIN_CONNECT --> STATE_MACHINE_FAILED
+
+  CONNECT_STAGE_NET_WAIT_CONNECT --> CONNECT_STAGE_NET_COMPLETE_CONNECT
+  CONNECT_STAGE_NET_WAIT_CONNECT --> STATE_MACHINE_FAILED
+
+  CONNECT_STAGE_NET_COMPLETE_CONNECT --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_NET_COMPLETE_CONNECT --> CONNECT_STAGE_READ_GREETING
+
+  CONNECT_STAGE_READ_GREETING --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_READ_GREETING --> CONNECT_STAGE_PARSE_HANDSHAKE
+
+  CONNECT_STAGE_PARSE_HANDSHAKE --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_PARSE_HANDSHAKE --> CONNECT_STAGE_ESTABLISH_SSL
+
+  CONNECT_STAGE_ESTABLISH_SSL --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_ESTABLISH_SSL --> CONNECT_STAGE_AUTHENTICATE
+
+  CONNECT_STAGE_AUTHENTICATE --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTHENTICATE --> CONNECT_STAGE_AUTH_BEGIN
+
+  CONNECT_STAGE_AUTH_BEGIN --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_BEGIN --> CONNECT_STAGE_AUTH_RUN_FIRST_AUTHENTICATE_USER
+
+  CONNECT_STAGE_AUTH_RUN_FIRST_AUTHENTICATE_USER --> CONNECT_STAGE_AUTH_HANDLE_FIRST_AUTHENTICATE_USER
+
+  CONNECT_STAGE_AUTH_HANDLE_FIRST_AUTHENTICATE_USER --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_HANDLE_FIRST_AUTHENTICATE_USER --> CONNECT_STAGE_AUTH_READ_CHANGE_USER_RESULT
+
+  CONNECT_STAGE_AUTH_READ_CHANGE_USER_RESULT --> CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST
+
+  CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST --> CONNECT_STAGE_AUTH_RUN_SECOND_AUTHENTICATE_USER
+  CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST --> CONNECT_STAGE_AUTH_INIT_MULTI_AUTH
+  CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST --> CONNECT_STAGE_AUTH_FINISH_AUTH
+
+  CONNECT_STAGE_AUTH_RUN_SECOND_AUTHENTICATE_USER --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_RUN_SECOND_AUTHENTICATE_USER --> CONNECT_STAGE_AUTH_HANDLE_SECOND_AUTHENTICATE_USER
+
+  CONNECT_STAGE_AUTH_HANDLE_SECOND_AUTHENTICATE_USER --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_HANDLE_SECOND_AUTHENTICATE_USER --> CONNECT_STAGE_AUTH_INIT_MULTI_AUTH
+  CONNECT_STAGE_AUTH_HANDLE_SECOND_AUTHENTICATE_USER --> CONNECT_STAGE_AUTH_FINISH_AUTH
+
+  CONNECT_STAGE_AUTH_INIT_MULTI_AUTH --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_INIT_MULTI_AUTH --> CONNECT_STAGE_AUTH_DO_MULTI_PLUGIN_AUTH
+
+  CONNECT_STAGE_AUTH_DO_MULTI_PLUGIN_AUTH --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_DO_MULTI_PLUGIN_AUTH --> CONNECT_STAGE_AUTH_HANDLE_MULTI_AUTH_RESPONSE
+
+  CONNECT_STAGE_AUTH_HANDLE_MULTI_AUTH_RESPONSE --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_HANDLE_MULTI_AUTH_RESPONSE --> CONNECT_STAGE_AUTH_INIT_MULTI_AUTH
+  CONNECT_STAGE_AUTH_HANDLE_MULTI_AUTH_RESPONSE --> CONNECT_STAGE_AUTH_FINISH_AUTH
+
+  CONNECT_STAGE_AUTH_FINISH_AUTH --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_AUTH_FINISH_AUTH --> CONNECT_STAGE_PREP_SELECT_DATABASE
+
+  CONNECT_STAGE_PREP_SELECT_DATABASE --> CONNECT_STAGE_COMPLETE
+  CONNECT_STAGE_PREP_SELECT_DATABASE --> CONNECT_STAGE_PREP_INIT_COMMANDS
+
+  CONNECT_STAGE_PREP_INIT_COMMANDS --> CONNECT_STAGE_COMPLETE
+  CONNECT_STAGE_PREP_INIT_COMMANDS --> CONNECT_STAGE_SEND_ONE_INIT_COMMAND
+
+  CONNECT_STAGE_SEND_ONE_INIT_COMMAND --> CONNECT_STAGE_SEND_ONE_INIT_COMMAND
+  CONNECT_STAGE_SEND_ONE_INIT_COMMAND --> STATE_MACHINE_FAILED
+  CONNECT_STAGE_SEND_ONE_INIT_COMMAND --> CONNECT_STAGE_COMPLETE
+
+  STATE_MACHINE_FAILED --> [*]
+  CONNECT_STAGE_COMPLETE --> [*]
+  CONNECT_STAGE_INVALID --> [*]
+  @enduml
+*/
+/* clang-format on */
+enum connect_stage {
+  /** MYSQL not valid or an unknown state */
+  CONNECT_STAGE_INVALID = 0,
+  /** not connected */
+  CONNECT_STAGE_NOT_STARTED,
+  /** begin connection to the server */
+  CONNECT_STAGE_NET_BEGIN_CONNECT,
+  /** wait for connection to be established */
+  CONNECT_STAGE_NET_WAIT_CONNECT,
+  /** init the local data structures post connect */
+  CONNECT_STAGE_NET_COMPLETE_CONNECT,
+  /** read the first packet */
+  CONNECT_STAGE_READ_GREETING,
+  /** parse the first packet */
+  CONNECT_STAGE_PARSE_HANDSHAKE,
+  /** tls establishment */
+  CONNECT_STAGE_ESTABLISH_SSL,
+  /** authentication phase */
+  CONNECT_STAGE_AUTHENTICATE,
+  /** determine the plugin to use */
+  CONNECT_STAGE_AUTH_BEGIN,
+  /** run first auth plugin */
+  CONNECT_STAGE_AUTH_RUN_FIRST_AUTHENTICATE_USER,
+  /** handle the result of the first auth plugin run */
+  CONNECT_STAGE_AUTH_HANDLE_FIRST_AUTHENTICATE_USER,
+  /** read the implied changed user auth, if any */
+  CONNECT_STAGE_AUTH_READ_CHANGE_USER_RESULT,
+  /** Check if server asked to use a different authentication plugin */
+  CONNECT_STAGE_AUTH_HANDLE_CHANGE_USER_REQUEST,
+  /** Start the authentication process again with the plugin which
+  server asked for */
+  CONNECT_STAGE_AUTH_RUN_SECOND_AUTHENTICATE_USER,
+  /** Start multi factor authentication */
+  CONNECT_STAGE_AUTH_INIT_MULTI_AUTH,
+  /** Final cleanup */
+  CONNECT_STAGE_AUTH_FINISH_AUTH,
+  /** Now read the results of the second plugin run */
+  CONNECT_STAGE_AUTH_HANDLE_SECOND_AUTHENTICATE_USER,
+  /** Invoke client plugins multi-auth authentication method */
+  CONNECT_STAGE_AUTH_DO_MULTI_PLUGIN_AUTH,
+  /** Handle response from client plugins authentication method */
+  CONNECT_STAGE_AUTH_HANDLE_MULTI_AUTH_RESPONSE,
+  /** Authenticated, set initial database if specified */
+  CONNECT_STAGE_PREP_SELECT_DATABASE,
+  /** Prepare to send a sequence of init commands. */
+  CONNECT_STAGE_PREP_INIT_COMMANDS,
+  /** Send an init command.  This is called once per init command until
+  they've all been run (or a failure occurs) */
+  CONNECT_STAGE_SEND_ONE_INIT_COMMAND,
+  /** Connected or no async connect in progress */
+  CONNECT_STAGE_COMPLETE
+};
 #endif
